//+------------------------------------------------------------------+
//|                                                           !FanMA |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006-2013"
#property version   "1.00"

#property indicator_chart_window
#property indicator_buffers	100
#property indicator_plots 	100

enum enProgression { Arithmetical, Fibonacci, Leonardo, Geometrical}; //     

input color Color=clrLimeGreen; //  
input enProgression Progress=Arithmetical; //   
input int nLine=10; //   
input int PeriodStart=2; //  
input double Koef=2; //  

input ENUM_MA_METHOD Method=MODE_EMA; // 
input ENUM_APPLIED_PRICE Price=PRICE_CLOSE; //  



//------------------------------------------------------------------	CIndicator
class CMA
{
	//  
public:
	int m_h; //  
	double m_d[]; // 
	int m_i; //    
	
	//   
	string m_smb; // 
	ENUM_TIMEFRAMES m_tf; // 
	int m_shift; // 
	int m_period; // 
	ENUM_MA_METHOD m_method; // 
	ENUM_APPLIED_PRICE m_price; //  
	
	//  
public:
	CMA() { m_h=0; ArraySetAsSeries(m_d, true); }
	~CMA() { if (m_h>0) IndicatorRelease(m_h); m_h=0; }
	void InitBuf(int i); //   
	//
public:
	bool Init(string smb, ENUM_TIMEFRAMES tf, int period, int shift, ENUM_MA_METHOD method, ENUM_APPLIED_PRICE price);
	bool GetData(int lim);
};
//------------------------------------------------------------------	InitBuf
void CMA::InitBuf(int i) //   
{
	m_i=i; SetIndexBuffer(m_i, m_d, INDICATOR_DATA); PlotIndexSetInteger(m_i, PLOT_DRAW_TYPE, DRAW_LINE);
	PlotIndexSetInteger(m_i, PLOT_LINE_COLOR, Color); PlotIndexSetInteger(m_i, PLOT_LINE_STYLE, STYLE_SOLID); PlotIndexSetInteger(m_i, PLOT_LINE_WIDTH, 1);
}
//------------------------------------------------------------------	InitMA
bool CMA::Init(string smb, ENUM_TIMEFRAMES tf, int period, int shift, ENUM_MA_METHOD method, ENUM_APPLIED_PRICE price)
{
	m_smb=smb; m_tf=tf; m_period=period; m_shift=shift; m_method=method; m_price=price;
	PlotIndexSetString(m_i, PLOT_LABEL, "MA("+string(m_period)+")");
	if (m_h>0) { IndicatorRelease(m_h); m_h=0; }
	m_h=iMA(m_smb, m_tf, m_period, 0, m_method, m_price);
	return(m_h>0); //  
}
//------------------------------------------------------------------	GetMA
bool CMA::GetData(int lim)
{
	int n=CopyBuffer(m_h, 0, 0, lim, d);
	if (n<lim) return(false);
	for (int i=0; i<lim; i++) m_d[i]=d[i];
	return(true);
}




CMA MAs[]; //  
double d[]; //      

//------------------------------------------------------------------	OnInit
int OnInit()
{
	ArrayResize(MAs, nLine); ArraySetAsSeries(d, true);
	for (int i=0; i<nLine; i++)
	{
		MAs[i].InitBuf(i); //    
		if (!MAs[i].Init(Symbol(), Period(), PeriodStart+NextPeriod(i), 0, Method, Price)) { Print("  ("+string(MAs[i].m_period)+")"); break; } //  
	}
	IndicatorSetString(INDICATOR_SHORTNAME, "MA_Fan("+string(nLine)+") "+EnumToString(Method)+"/"+EnumToString(Price));
	return(INIT_SUCCEEDED);
}
//------------------------------------------------------------------	OnCalculate
int OnCalculate(const int rates_total, const int prev_calculated, const datetime &time[], const double &open[], const double &high[], const double &low[], const double &close[], const long &tick_volume[], const long &volume[], const int &spread[])
{
	int lim=rates_total-prev_calculated; if (lim>1) lim=rates_total; else lim++; //     
	bool bOk=true; for (int i=0; i<nLine; i++) if (!MAs[i].GetData(lim)) bOk=false; //    
	if (bOk) return(rates_total); else return(prev_calculated);
}
//------------------------------------------------------------------	NextPeriod
int NextPeriod(int s)
{
	double p=0;
	switch(Progress)
	{
	case Arithmetical: p=Koef*s; break;
	case Fibonacci: { double p0=0, p1=1; for (int i=2; i<s; i++) { p=p0+p1; p0=p1; p1=p; } } break;
	case Leonardo: { double p0=1, p1=1; for (int i=2; i<s; i++) { p=p0+p1+1; p0=p1; p1=p; } } break;
	case Geometrical: p=MathPow(Koef, s); break;
	}
	return(int(p));
}
